/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2011  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 3.82t                                            *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : RTOSInit_OMAP3517.c
Purpose : Initializes and handles the hardware for the OS as far
          as required by the OS.
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "RTOS.h"

/*********************************************************************
*
*       Configuration
*
*********************************************************************/

#define ALLOW_NESTED_INTERRUPTS 0   // Caution: Nesting interrupts will cause higher stack load on system stack CSTACK

/*********************************************************************
*
*       Clock frequency settings
*/
#ifndef   OS_FSYS                   /* CPU main clock frequency      */
  #define OS_FSYS 600000000uL
#endif

#ifndef   OS_PCLK_TIMER             /* Peripheral clock for timer   */
  #define OS_PCLK_TIMER 13000000uL      
#endif                              

#ifndef   OS_PCLK_UART              /* Peripheral clock for UART    */
  #define OS_PCLK_UART 48000000uL   /* May vary from CPU clock      */
#endif                              /* depending on CPU             */

#ifndef   OS_TICK_FREQ
  #define OS_TICK_FREQ (1000)
#endif

/*********************************************************************
*
*       Configuration of communication to OSView
*/
#ifndef   OS_VIEW_ENABLE            // Global enable of communication
  #define OS_VIEW_ENABLE    (1)     // Default: on
#endif

/*********************************************************************
*
*       UART settings for OSView
*       If you do not want (or can not due to hardware limitations)
*       to dedicate a UART to OSView, please define it to be -1
*       Currently the standard code enables UART 3 per default
*/
#ifndef   OS_UART
  #define OS_UART (3)
#endif

#ifndef   OS_BAUDRATE
  #define OS_BAUDRATE (38400)
#endif

/****** End of configuration settings *******************************/

#define OS_UART_USED (OS_VIEW_ENABLE && (OS_UART == 3))

/*********************************************************************
*
*       Local defines (sfrs used in RTOSInit.c)
*
**********************************************************************
*/
/* Clock and Power registers */
#define CM_FCLKEN1_CORE                (*(volatile OS_U32*)(0x48004A00)) 
#define CM_ICLKEN1_CORE                (*(volatile OS_U32*)(0x48004A10))
#define CM_FCLKEN_PER                  (*(volatile OS_U32*)(0x48005000))
#define CM_FCLKEN_WKUP                 (*(volatile OS_U32*)(0x48004C00)) 
#define CM_ICLKEN_WKUP                 (*(volatile OS_U32*)(0x48004C10))
#define CM_ICLKEN_PER                  (*(volatile OS_U32*)(0x48005010))
#define CM_AUTOIDLE1_CORE              (*(volatile OS_U32*)(0x48004A30)) 
#define CM_CLKSEL2_PLL_MPU             (*(volatile OS_U32*)(0x48004944)) 
#define CM_CLKSEL1_PLL_MPU             (*(volatile OS_U32*)(0x48004940)) 
#define CM_CLKEN_PLL_MPU               (*(volatile OS_U32*)(0x48004904)) 
#define CM_CLKSEL1_PLL                 (*(volatile OS_U32*)(0x48004D40)) 
#define CM_CLKSEL2_PLL                 (*(volatile OS_U32*)(0x48004D44)) 
#define CM_CLKSEL3_PLL                 (*(volatile OS_U32*)(0x48004D48)) 
#define CM_CLKSEL_CORE                 (*(volatile OS_U32*)(0x48004A40)) 
#define CM_CLKEN_PLL                   (*(volatile OS_U32*)(0x48004D00)) 
#define CM_CLKSEL_WKUP                 (*(volatile OS_U32*)(0x48004C40)) 
#define CM_IDLEST_WKUP                 (*(volatile OS_U32*)(0x48004C20))

/* Watchdog control registers */
#define WD_SYSSTATUS                   (*(volatile OS_U32*)(0x48314014))
#define WWPS                           (*(volatile OS_U32*)(0x48314034))
#define WSPR                           (*(volatile OS_U32*)(0x48314048))

/* Interrupt Vector Controller */
#define MPUINTC_BASEADDR               0x48200000
#define INTCPS_SYSCONFIG               (*(volatile OS_U32*)(MPUINTC_BASEADDR + 0x00000010))
#define INTCPS_SYSSTATUS               (*(volatile OS_U32*)(MPUINTC_BASEADDR + 0x00000014))
#define INTCPS_SIR_IRQ                 (*(volatile OS_U32*)(MPUINTC_BASEADDR + 0x00000040))
#define INTCPS_SIR_FIQ                 (*(volatile OS_U32*)(MPUINTC_BASEADDR + 0x00000044))
#define INTCPS_CONTROL                 (*(volatile OS_U32*)(MPUINTC_BASEADDR + 0x00000048))
#define INTCPS_PROTECTION              (*(volatile OS_U32*)(MPUINTC_BASEADDR + 0x0000004C))
#define INTCPS_MIR0_CLR                (*(volatile OS_U32*)(MPUINTC_BASEADDR + 0x00000088))
#define INTCPS_MIR1_CLR                (*(volatile OS_U32*)(MPUINTC_BASEADDR + 0x000000A8))
#define INTCPS_MIR2_CLR                (*(volatile OS_U32*)(MPUINTC_BASEADDR + 0x000000C8))
#define INTCPS_MIR0_SET                (*(volatile OS_U32*)(MPUINTC_BASEADDR + 0x0000008C))
#define INTCPS_MIR1_SET                (*(volatile OS_U32*)(MPUINTC_BASEADDR + 0x000000AC))
#define INTCPS_MIR2_SET                (*(volatile OS_U32*)(MPUINTC_BASEADDR + 0x000000CC))
#define INTCPS_ILR_ADDR                (MPUINTC_BASEADDR + 0x00000100)

#define NUM_INT_VECTORS                96

/* Timer 1 register */
#define GPTIMER1_BASEADDR              0x48318000
#define TIOCP_CFG                      (*(volatile OS_U32*)(GPTIMER1_BASEADDR + 0x010))
#define TISTAT                         (*(volatile OS_U32*)(GPTIMER1_BASEADDR + 0x014))
#define TISR                           (*(volatile OS_U32*)(GPTIMER1_BASEADDR + 0x018))
#define TIER                           (*(volatile OS_U32*)(GPTIMER1_BASEADDR + 0x01C))
#define TCLR                           (*(volatile OS_U32*)(GPTIMER1_BASEADDR + 0x024))
#define TCRR                           (*(volatile OS_U32*)(GPTIMER1_BASEADDR + 0x028))
#define TLDR                           (*(volatile OS_U32*)(GPTIMER1_BASEADDR + 0x02C))
#define TMAR                           (*(volatile OS_U32*)(GPTIMER1_BASEADDR + 0x038))
#define TPIR                           (*(volatile OS_U32*)(GPTIMER1_BASEADDR + 0x048))
#define TNIR                           (*(volatile OS_U32*)(GPTIMER1_BASEADDR + 0x04C))

/* Uart 3 */
#define UART3_BASEADDR                 0x49020000
#define DLL_REG                        (*(volatile OS_U16*)(UART3_BASEADDR + 0x000))
#define RHR_REG                        (*(volatile OS_U16*)(UART3_BASEADDR + 0x000))
#define THR_REG                        (*(volatile OS_U16*)(UART3_BASEADDR + 0x000))
#define DLH_REG                        (*(volatile OS_U16*)(UART3_BASEADDR + 0x004))
#define IER_REG                        (*(volatile OS_U16*)(UART3_BASEADDR + 0x004))
#define IIR_REG                        (*(volatile OS_U16*)(UART3_BASEADDR + 0x008))
#define EFR_REG                        (*(volatile OS_U16*)(UART3_BASEADDR + 0x008))
#define LCR_REG                        (*(volatile OS_U16*)(UART3_BASEADDR + 0x00C))
#define MCR_REG                        (*(volatile OS_U16*)(UART3_BASEADDR + 0x010))
#define SPR_TLR_REG                    (*(volatile OS_U16*)(UART3_BASEADDR + 0x01C))
#define SCR_REG                        (*(volatile OS_U16*)(UART3_BASEADDR + 0x040))
#define SYSC_REG                       (*(volatile OS_U16*)(UART3_BASEADDR + 0x054))
#define SYSS_REG                       (*(volatile OS_U16*)(UART3_BASEADDR + 0x058))
#define MDR1_REG                       (*(volatile OS_U16*)(UART3_BASEADDR + 0x020))
#define CONTROL_PADCONF_UART3_RTS_SD   (*(volatile OS_U16*)(0x4800219C ))
#define CONTROL_PADCONF_UART3_TX_IRTX  (*(volatile OS_U16*)(0x480021A0))

/* Neon coprocessor  clock and power registers */
#define PM_WKDEP_NEON                  (*(volatile OS_U32*)(0x483073C8))
#define PM_PWSTCTRL_NEON               (*(volatile OS_U32*)(0x483073E0))
#define CM_IDLEST_NEON                 (*(volatile OS_U32*)(0x48005320))
#define CM_CLKSTCTRL_NEON              (*(volatile OS_U32*)(0x48005348))

#define OVF_IT_ENA                     (1uL << 1)            // Overflow interrupt enable
#define OVF_IT_FLAG                    (1uL << 1)            // Overflow interrupt flag
#define LCR_CONF_MODE_A                0x80                  // Switch device to configuration mode A  
#define LCR_CONF_MODE_B                0xBF                  // Switch device to configuration mode B      
#define MDR1_MODE_SELECT_UART16        0x00                  // UART mode selection UART 16x mode          
#define FCR_FIFO_EN                    (1uL << 0)            // Set the FIFO enable bit, enables TX/RX FIFO
#define FCR_RX_FIFO_CLEAR              (1uL << 11)           // Clears the receive FIFO buffer
#define FCR_TX_FIFO_CLEAR              (1uL << 2)            // Clears the transmit FIFO buffer
#define FCR_DMA_MODE                   (1uL << 3)            // Enables DMA mode
#define FCR_TXFIFO_TRIG_LSB(x)         (x << 4)              // Two LSB bits of transmit FIFO trigger level
#define FCR_RXFIFO_TRIG_LSB(x)         (x << 6)              // Two LSB bits of receive FIFO trigger level
#define TLR_TX_FIFO_TRIG_DMA(x)        (x & 0x0FL )          // Two LSB bits of transmit FIFO trigger level  
#define TLR_RX_FIFO_TRIG_DMA(x)        (x << 4)              // Two LSB bits of receive FIFO trigger level
#define MCR_TCR_TLR                    6
#define SCR_TX_TRIG_GRANU1             6                     // Set Granularity of TX FIFO trigger level to 1 
#define SCR_RX_TRIG_GRANU1             7                     // Set Granularity of RX FIFO trigger level to 1 
#define IER_RHR_IT                     0                     // Enable transmit holding register interrupt 
#define IER_THR_IT                     1                     // Enable transmit holding register interrupt 
#define IIR_IT_TYPE_MASK               (0x1fuL << 1)         // Mask Bit for reading out interrupt type
#define IIR_IT_TYPE_THR                (0x01uL << 1)         // Transmit holding register interrupt
#define IIR_IT_TYPE_RHR                (0x02uL << 1)         // Receive holding register interrupt
#define IIR_IT_TYPE_RXTIMEOUT          (0x06uL << 1)         // Receive Timeout interrupt

#define TIMER1_IRQ_ID                  37 
#define TIMER1_IRQ_PRIO                62

#define UART3_IRQ_ID                   74
#define UART3_IRQ_PRIO                 63   

#define TBL_START_ADDR                 ((OS_U32*)0x4020C000) // The Translation table is stored at a fixed address at beginning of last 16Kbyte of internal SRAM

#define OS_TIMER_RELOAD (OS_PCLK_TIMER/OS_TICK_FREQ)
#if (OS_TIMER_RELOAD >= 0x100000000)
  #error "Timer 1 can not be used, please check configuration"
#endif

#define BRR_VAL (OS_PCLK_UART/16L/OS_BAUDRATE)
#if BRR_VAL > 0xFFFF
  #error "UART can not be used with standard initialization."
#endif

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/
static OS_ISR_HANDLER* aIRQTable[NUM_INT_VECTORS];             // IRQ table for Interrupt service routines

/*********************************************************************
*
*       MMU and cache configuration
*/
unsigned int *_pTranslationTable;        
  
/*********************************************************************
*
*       Prototypes
*
**********************************************************************
*/
static void _OS_ISR_Undefined(void);

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

/*********************************************************************
*
*       _OS_Systick
*
* Function description
*   This is the code that gets called when the processor receives a
*   _SysTick exception. SysTick is used as OS timer tick.
*
* NOTES:
*   (1) It has to be inserted in the interrupt vector table, if RAM
*       vectors are not used. Therefore is is declared public
*/
static void _OS_Systick(void) {
  TISR |= OVF_IT_FLAG;   // Reset pending flag
  OS_HandleTick();
}

/*********************************************************************
*
*       _InitVIC()
*
*       Initialize interrupt controller by setting default vectors
*       and clearing all interrupts
*/
static void _InitVIC(void) {
  int i;
  INTCPS_SYSCONFIG = 0x02;                    // Reset interrupt controller;
  while ((INTCPS_SYSSTATUS & 0x01) == 0x00);  // Wait until reset is finished
  __asm__("ldr r0, =__ivt_start");            // Get start addr of interrupt vector table
  __asm__("mcr p15, 0, r0, c12, c0, 0");      // Set Vector Base addr
  for (i = 0; i < NUM_INT_VECTORS; i++) {
    aIRQTable[i] = (OS_ISR_HANDLER*) _OS_ISR_Undefined;
  }
}

/*********************************************************************
*
*       _WatchdogDisable()
*
*       Disables all watchdogs
*/
static void _WatchdogDisable(void) {
  OS_U32 r;
  //
  // Disable watchdog
  //
  // Enable Interface Clock
  CM_ICLKEN_WKUP = 0x20uL;
  // Enable functional clock
  CM_FCLKEN_WKUP = 0x20;

  // Check that module is IDLE
  do {
    r = CM_IDLEST_WKUP;
  } while ((r & 0x20) != 0);

  // Disable watchdog timer
  WSPR = 0x0000AAAA;
  do {
    r = WWPS;
  } while ((r & 0x10) != 0);

  WSPR = 0x00005555;
  do {
    r = WWPS;
  } while ((r & 0x10) != 0);

  // Check if the watchdog timer is running
  r = WSPR;
  if (r != 0x00005555) {
    // Disable Watchdog 2
    // Wait until reset complete
    do {
      r = WD_SYSSTATUS;
    } while ((r & 0x01) == 0);

    // Disable 32Khz watchdog timer
    WSPR = 0x0000AAAA;

    do {
      r = WWPS;
    } while ((r & 0x10) != 0);

    // Disable 32Khz watchdog timer
    WSPR = 0x00005555;

    do {
      r = WWPS;
    } while ((r & 0x10) != 0);
  }
}

/*********************************************************************
*
*       _InitPLL()
*
*       Initialize PLL
*/
static void _InitPLL(void) {
  int i;
  //
  // Now speed up DPLL1 (MPU)
  //
  CM_CLKSEL2_PLL_MPU = 0x00000001;     // Use CLKOUT / 1
  CM_CLKSEL1_PLL_MPU = 0x0012580C;     // Bypass = CORE_CLK/2; Multiplier=600; divider=12(+1) == 600MHz
  CM_CLKEN_PLL_MPU   = 0x00000037;     // MPU_DPLL_FREQSEL = 0x7, EN_MPU_DPLL = 0x7 Lock mode
  for (i = 0; i < 0xFFFF; i++);        // Wait a short time
  //
  // Set up DPLL3 clock
  //
  CM_CLKSEL1_PLL = 0x094C0C00;
  //
  // Set up DPLL4 clock
  //
  CM_CLKSEL2_PLL = 0x0001b00c;
  //
  // Set up 96MHz clock
  //
  CM_CLKSEL3_PLL = 0x00000009;
  //
  // Lock the DPLL3 and DPLL4
  //
  CM_FCLKEN1_CORE = 0x0003A000;
  CM_ICLKEN1_CORE = 0x0003A042;
  CM_CLKSEL_CORE  = 0x0000030A;
  CM_CLKEN_PLL    = 0x00370037;
  for (i = 0; i < 0xFFFF; i++);        // Wait a short time
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_InitHW()
*
*       Initialize the hardware (timer) required for the OS to run.
*       May be modified, if an other timer should be used
*/
void OS_InitHW(void) {
  OS_IncDI();
  OS_ARM_CACHE_Sync();
  _InitPLL();
  _InitVIC();
  //
  // Enable Neon Coprocessor
  //
  OS_NEON_Enable();
  PM_WKDEP_NEON     = 0x00;                      // Bit 1: 0x0: NEON domain is independent of MPU domain wake-up
  PM_PWSTCTRL_NEON  = 0x07;                      // Bit 2: 0x1: Logic is always retained when domain is in RETENTION state / Bits 0:1: POWERSTATE=0x3: ON State 
  CM_CLKSTCTRL_NEON = 0x03;                      // Automatic transition is enabled. Transition is supervised by the hardware
  // 
  // Enable clock and power for Timer1 
  //
  CM_FCLKEN_WKUP |= (1uL << 0);                  // Enable function clock
  CM_ICLKEN_WKUP |= (1uL << 0);                  // Enable interface clock
  CM_CLKSEL_WKUP |= (1uL << 0);                  // Set SysClk as clock source
  // 
  // Init GPTIMER1 as System tick timer
  //
  TLDR  = 0xFFFFFFFF - OS_TIMER_RELOAD + 1;       // Timer reload register
  TCRR  = 0xFFFFFFFF - OS_TIMER_RELOAD + 1;       // Timer value
  TIER |= OVF_IT_ENA;                             // Enable Overflow interrupt
  TCLR  = (1uL << 1)                              // Auto reload mode
       |  (1uL << 0);                             // Start Timer
  OS_ARM_InstallISRHandler (TIMER1_IRQ_ID, (OS_ISR_HANDLER*)_OS_Systick);
  OS_ARM_ISRSetPrio(TIMER1_IRQ_ID, TIMER1_IRQ_PRIO);
  OS_ARM_EnableISR(TIMER1_IRQ_ID);
  OS_COM_Init();   
  OS_DecRI();
}

/*********************************************************************
*
*       Idle loop  (OS_Idle)
*
*       Please note:
*       This is basically the "core" of the idle loop.
*       This core loop can be changed, but:
*       The idle loop does not have a stack of its own, therefore no
*       functionality should be implemented that relies on the stack
*       to be preserved. However, a simple program loop can be programmed
*       (like toggeling an output or incrementing a counter)
*/
void OS_Idle(void) {     // Idle loop: No task is ready to execute
  while (1) {
  }
}

/*********************************************************************
*
*       Get time [cycles]
*
*       This routine is required for high
*       resolution time measurement functions.
*       It returns the system time in timer clock cycles.
*/
OS_U32 OS_GetTime_Cycles(void) {
  OS_U32 time;
  int    count;
  time  = OS_Time;
  count = TCRR - (0xFFFFFFFF - OS_TIMER_RELOAD + 1);
  if ((TISR & OVF_IT_FLAG) != 0) {  // Missed a counter interrupt? Then adjust time
    time++;
    count = TCRR - (0xFFFFFFFF - OS_TIMER_RELOAD + 1);
  }
  return (OS_PCLK_TIMER/OS_TICK_FREQ) * time + count;
}

/*********************************************************************
*
*       OS_ConvertCycles2us
*
*       Convert Cycles into micro seconds.
*
*       If your clock frequency is not a multiple of 1 MHz,
*       you may have to modify this routine in order to get proper
*       diagnostics.
*
*       This routine is required for profiling or high resolution time
*       measurement only. It does not affect operation of the OS.
*/
OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  return Cycles/(OS_PCLK_TIMER/1000000);
}
   
/*********************************************************************
*
*       Communication for OSView via UART (optional)
*
**********************************************************************
*/
#if OS_UART_USED

/*********************************************************************
*
*       OS_COM_IsrHandler() 
*
*       Uart Interrupt Handler
*/
void OS_COM_IsrHandler(void) {
  switch ( IIR_REG & IIR_IT_TYPE_MASK ) {
    case IIR_IT_TYPE_THR:
      if (OS_OnTx()) {                   // No more characters to send ?
        IER_REG &= ~(1uL << IER_THR_IT); // Disable Tx Interrupt
      }
      break;
    case IIR_IT_TYPE_RHR:
      OS_OnRx(RHR_REG); 
      break;
    default:
      break;
  }
}

/*********************************************************************
*
*       OS_COM_Send1() 
*
*       Send one byte to embOSView.
*       NEVER call this function from your application
*/
void OS_COM_Send1(OS_U8 c) {
  IER_REG |= (1uL << IER_THR_IT);  // Enable Tx interrupt
  THR_REG = c;
}

/*********************************************************************
*
*       OS_COM_Init()
*
*       Initialize UART for communication with embOSView
*/
void OS_COM_Init(void) {
  volatile int i;
  CM_FCLKEN_PER     |= (1uL << 11);
  CM_ICLKEN_PER     |= (1uL << 11);
  for (i = 0; i < 0xFFFF; i++);                  // Wait a short time
  SYSC_REG          |= (1uL << 1);               // Initiate a software reset
  while ((SYSS_REG & (1uL << 0)) == 0);          // Wait for the end of the reset operation
  MDR1_REG           = MDR1_MODE_SELECT_UART16;
  LCR_REG            = LCR_CONF_MODE_B;
  EFR_REG           |= (1uL << 4);
  LCR_REG            = LCR_CONF_MODE_A;
  MCR_REG           |= (1uL << MCR_TCR_TLR);
  SCR_REG           |= (1uL << SCR_TX_TRIG_GRANU1) | (1uL << SCR_RX_TRIG_GRANU1);
  IIR_REG            = FCR_FIFO_EN | FCR_RX_FIFO_CLEAR | FCR_TX_FIFO_CLEAR | FCR_TXFIFO_TRIG_LSB(0) | FCR_RXFIFO_TRIG_LSB(1);
  SPR_TLR_REG        = TLR_TX_FIFO_TRIG_DMA(2) | TLR_RX_FIFO_TRIG_DMA(0);
  MDR1_REG           = MDR1_MODE_SELECT_UART16;         
  DLL_REG            = (OS_U8)(BRR_VAL & 0xFF); 
  DLH_REG            = (OS_U8)((BRR_VAL >> 8) & 0xFF);
  LCR_REG            = 0x13;                     // Set uart to 8N1
  IER_REG           |= (1uL << IER_RHR_IT);      // Enable Rx interrupt
  CONTROL_PADCONF_UART3_TX_IRTX &= ~0x03;        // Clear the three mode bits [0..2] to set mode 0
  CONTROL_PADCONF_UART3_RTS_SD &= ~0x030000;     // Clear the three mode bits [16..18] to set mode 0
  OS_ARM_InstallISRHandler (UART3_IRQ_ID, (OS_ISR_HANDLER*)OS_COM_IsrHandler);
  OS_ARM_ISRSetPrio(UART3_IRQ_ID, UART3_IRQ_PRIO);
  OS_ARM_EnableISR(UART3_IRQ_ID);
}
#else
/*********************************************************************
*
*       Communication for embOSView not selected
*
**********************************************************************
*/
void OS_COM_Init(void) {}
void OS_COM_Send1(OS_U8 c) {
  OS_USEPARA(c);
  OS_COM_ClearTxActive();  /* Free OS transmitter buffer */
}

/*********************************************************************
*
*       OS_COM_IsrHandler
*
* Function description
*   The communication interrupt handler for UART communication
*   to embOSView.
*
* NOTES:
*   (1) It has to be inserted in the interrupt vector table, if RAM
*       vectors are not used. Therefore is is declared public
*/
void OS_COM_IsrHandler(void) {
  while(1);
}
#endif

/****** Final check of configuration ********************************/

#ifndef OS_UART_USED
  #error "OS_UART_USED has to be defined"
#endif

/*********************************************************************
*
*       OS interrupt handler and ISR specific functions
*
**********************************************************************
*/

/*********************************************************************
*
*       _OS_ISR_Undefined
*
*       Is called when an uninstalled interrupt was detected
*       As interrupt pending condition of peripherals has to be reset,
*       program will not continue when interrupt is ignored.
*/
static void _OS_ISR_Undefined(void) {
  volatile int Dummy;
  Dummy = 1;
  /* You may set a breakpoint here to detect undefined interrupts */
  while (Dummy > 0) {
  }
}

/*********************************************************************
*
*       OS_irq_handler
*
*       Detect reason for IRQ and call corresponding service routine.
*       OS_irq_handler is called from IRQ_Handler function
*       found in rtos.s
*/
void OS_irq_handler(void) {
  OS_ISR_HANDLER* pISR;
  int ActiveInterrupt;
#if ALLOW_NESTED_INTERRUPTS
  OS_EnterNestableInterrupt();                // Now interrupts may be reenabled. If nesting should be allowed
#else
  OS_EnterInterrupt();                        // Inform OS that interrupt handler is running
#endif
  //
  // Read active interrupt register and branch to Irq service routine
  //
  ActiveInterrupt = INTCPS_SIR_IRQ;
  pISR = (OS_ISR_HANDLER *)aIRQTable[ActiveInterrupt];
  pISR();
  OS_DI();                                    // Disable interrupts and unlock
  INTCPS_CONTROL = 0x01;                      // Reset NVIC
#if ALLOW_NESTED_INTERRUPTS
  OS_LeaveNestableInterrupt();                // Leave nestable interrupt, perform task switch if required
#else
  OS_LeaveInterrupt();                        // Leave interrupt, perform task switch if required
#endif
}

/*********************************************************************
*
*       OS_ARM_InstallISRHandler
*/
OS_ISR_HANDLER* OS_ARM_InstallISRHandler (int ISRIndex, OS_ISR_HANDLER* pISRHandler) {
  OS_ISR_HANDLER*  pOldHandler;
  OS_ISR_HANDLER** papISR;

#if OS_DEBUG
  if (ISRIndex >= NUM_INT_VECTORS) {
    OS_Error(OS_ERR_ISR_INDEX);
    return NULL;
  }
#endif
  OS_DI();
  papISR = (OS_ISR_HANDLER**) aIRQTable;
  pOldHandler          = *(papISR + ISRIndex);
  *(papISR + ISRIndex) = pISRHandler;
  OS_RestoreI();
  return pOldHandler;
}

/*********************************************************************
*
*       OS_ARM_EnableISR
*/
void OS_ARM_EnableISR(int ISRIndex) {
#if OS_DEBUG
  if (ISRIndex >= NUM_INT_VECTORS) {
    OS_Error(OS_ERR_ISR_INDEX);
    return;
  }
#endif
  OS_DI();
  if (ISRIndex < 32) {
    INTCPS_MIR0_CLR = (1uL << (ISRIndex));
  } else if (ISRIndex < 64) {
    INTCPS_MIR1_CLR = (1uL << (ISRIndex - 32));
  } else if (ISRIndex < 96) {
    INTCPS_MIR2_CLR = (1uL << (ISRIndex - 64));
  }
  OS_RestoreI();
}

/*********************************************************************
*
*       OS_ARM_DisableISR
*/
void OS_ARM_DisableISR(int ISRIndex) {
#if OS_DEBUG
  if (ISRIndex >= NUM_INT_VECTORS) {
    OS_Error(OS_ERR_ISR_INDEX);
    return;
  }
#endif
  OS_DI();
  if (ISRIndex < 32) {
    INTCPS_MIR0_SET = (1uL << (ISRIndex));
  } else if (ISRIndex < 64) {
    INTCPS_MIR1_SET = (1uL << (ISRIndex - 32));
  } else if (ISRIndex < 96) {
    INTCPS_MIR2_SET = (1uL << (ISRIndex - 64));
  }
  OS_RestoreI();
}

/*********************************************************************
*
*       OS_ARM_ISRSetPrio
*/
int OS_ARM_ISRSetPrio(int ISRIndex, int Prio) {
  OS_U32* pPrio;
  int     OldPrio;
#if DEBUG
  if ((unsigned)ISRIndex >= NUM_INT_VECTORS) {
    OS_Error(OS_ERR_ISR_INDEX);
    return 0;
  }
#endif
  OS_DI();
  pPrio = (OS_U32*)INTCPS_ILR_ADDR;
  OldPrio = pPrio[ISRIndex];
  pPrio[ISRIndex] = ((Prio & 0x3F) << 2);
  OS_RestoreI();
  return OldPrio;
}

/*********************************************************************
*
*       __low_level_init()
*
*       Initialize MMU
*
*       This function is called during startup and
*/
void __low_level_init(void) {
  _WatchdogDisable();
  //
  // Init MMU and caches. This defines the virtual memory map, which is used during execution.
  // Memory mapping should be complete, meaning 4096 entries.
  // Code below fills in ascending VAddr order
  //           
  _pTranslationTable = TBL_START_ADDR;
  OS_ARM_MMU_InitTT(_pTranslationTable);
  //                                          Mode                        VAddr  PAddr  Size[MB]
  OS_ARM_MMU_AddTTEntries(_pTranslationTable, OS_ARM_CACHEMODEA7_ILLEGAL, 0x000, 0x000, 0x402);    // Unmapped area
  OS_ARM_MMU_AddTTEntries(_pTranslationTable, OS_ARM_CACHEMODEA7_C_B,     0x402, 0x402, 0x001);    // internal SRAM (64kByte), 1 MB (program area) 
  OS_ARM_MMU_AddTTEntries(_pTranslationTable, OS_ARM_CACHEMODEA7_ILLEGAL, 0x403, 0x403, 0x07D);    // Unmapped area
  OS_ARM_MMU_AddTTEntries(_pTranslationTable, OS_ARM_CACHEMODEA7_NC_NB,   0x480, 0x480, 0x080);    // SFRs, non cacheable, non bufferable
  OS_ARM_MMU_AddTTEntries(_pTranslationTable, OS_ARM_CACHEMODEA7_ILLEGAL, 0x500, 0x500, 0x310);    // Unmapped area
  OS_ARM_MMU_AddTTEntries(_pTranslationTable, OS_ARM_CACHEMODEA7_C_B,     0x810, 0x810, 0x001);    // external SDRAM (128 MByte), 1 MB (program area) 
  OS_ARM_MMU_AddTTEntries(_pTranslationTable, OS_ARM_CACHEMODEA7_ILLEGAL, 0x811, 0x811, 0x7EE);    // Unmapped area
  OS_ARM_MMU_Enable      (_pTranslationTable);
  OS_ARM_ICACHE_Enable();
  OS_ARM_DCACHE_Enable();
}

/****** End Of File *************************************************/
