/*
 * The authors hereby grant permission to use, copy, modify, distribute,
 * and license this software and its documentation for any purpose, provided
 * that existing copyright notices are retained in all copies and that this
 * notice is included verbatim in any distributions. No written agreement,
 * license, or royalty fee is required for any of the authorized uses.
 * Modifications to this software may be copyrighted by their authors
 * and need not follow the licensing terms described here, provided that
 * the new terms are clearly indicated on the first page of each file where
 * they apply.
 *
 *
 * SEGGER Microcontroller GmbH & Co KG:
 *  2011/06/01: Stack initialization added 
 */

        ARM_MODE_USER  = 0x10        /* Normal User Mode                             */ 
        ARM_MODE_FIQ   = 0x11        /* FIQ Fast Interrupts Mode                     */
        ARM_MODE_IRQ   = 0x12        /* IRQ Standard Interrupts Mode                 */
        ARM_MODE_SVC   = 0x13        /* Supervisor Interrupts Mode                   */
        ARM_MODE_ABORT = 0x17        /* Abort Processing memory Faults Mode          */
        ARM_MODE_UNDEF = 0x1B        /* Undefined Instructions Mode                  */
        ARM_MODE_SYS   = 0x1F        /* System Running in Priviledged Operating Mode */
        ARM_MODE_MASK  = 0x1F
       
        I_BIT          = 0x80        /* Disables IRQ when I bit is set               */
        F_BIT          = 0x40        /* Disables FIQ when F bit is set               */

	.section .text.cs3.init,"ax",%progbits
#if defined(__thumb2__) || defined(__ARM_ARCH_6M__)
#define THUMB 1
	.code 16
	.thumb_func
#else
	.code 32
#endif
	.globl _start
	.type _start, %function
_start:
#if THUMB
	ldr	r1,=__cs3_stack
	mov	sp,r1
	ldr	r1,=__cs3_start_c
	bx	r1
#else
        /*
         *  Disable data and instruction cache and MMU
         */
        mrc     p15, 0, r1, c1, c0, 0
        bic     r0, r1, #0x1000                            /* Clear I, C and M bit */
        bic     r0, r0, #0x5
        mcr     p15, 0, r0, c1, c0, 0

        /*
         * Setup a stack for each mode
         */    
        msr   CPSR_c, #ARM_MODE_UNDEF | I_BIT | F_BIT   /* Undefined Instruction Mode */     
        ldr   sp, =__stack_und_end__
       
        msr   CPSR_c, #ARM_MODE_ABORT | I_BIT | F_BIT   /* Abort Mode */
        ldr   sp, =__stack_abt_end__
       
        msr   CPSR_c, #ARM_MODE_FIQ   | I_BIT | F_BIT   /* FIQ Mode */   
        ldr   sp, =__stack_fiq_end__
       
        msr   CPSR_c, #ARM_MODE_IRQ   | I_BIT | F_BIT   /* IRQ Mode */   
        ldr   sp, =__stack_irq_end__
       
        msr   CPSR_c, #ARM_MODE_SVC   | I_BIT | F_BIT   /* Supervisor Mode */
        ldr   sp, =__stack_svc_end__

        /* 
         * Enable Neon coprocessor very early to ensure that we 
         * can use floating point instructions
         */
        ldr   r0, = OS_NEON_Enable
        mov   lr, pc
        bx    r0

        /* 
         * Now enter embOS __low_level_init function, 
         * which does MMU initialization
         */
        ldr     r0,=__low_level_init
        mov     lr, pc
        bx      r0

        /* 
         * Now enter highh level startup code, 
         * which then calls main().
         */
        ldr   r0, = __cs3_start_c
        mov   lr, pc
        bx    r0
#endif
	.pool
